package sun.awt.im;

import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.text.AttributedCharacterIterator;
import java.awt.event.InputMethodEvent;
import java.awt.Graphics2D;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.im.InputMethodRequests;
import java.awt.Dimension;
import java.awt.Color;
import javax.swing.border.LineBorder;
import java.awt.Toolkit;
import javax.swing.JFrame;
import java.awt.font.TextHitInfo;
import java.awt.font.TextLayout;
import java.awt.event.InputMethodListener;
import javax.swing.JPanel;

public final class CompositionArea extends JPanel implements InputMethodListener
{
    private CompositionAreaHandler handler;
    private TextLayout composedTextLayout;
    private TextHitInfo caret;
    private JFrame compositionWindow;
    private static final int TEXT_ORIGIN_X = 5;
    private static final int TEXT_ORIGIN_Y = 15;
    private static final int PASSIVE_WIDTH = 480;
    private static final int WIDTH_MARGIN = 10;
    private static final int HEIGHT_MARGIN = 3;
    private static final long serialVersionUID = -1057247068746557444L;
    
    CompositionArea() {
        this.caret = null;
        this.compositionWindow = (JFrame)InputMethodContext.createInputMethodWindow(Toolkit.getProperty("AWT.CompositionWindowTitle", "Input Window"), null, true);
        this.setOpaque(true);
        this.setBorder(LineBorder.createGrayLineBorder());
        this.setForeground(Color.black);
        this.setBackground(Color.white);
        this.enableInputMethods(true);
        this.enableEvents(8L);
        this.compositionWindow.getContentPane().add(this);
        this.compositionWindow.addWindowListener(new CompositionArea.FrameWindowAdapter());
        this.addInputMethodListener(this);
        this.compositionWindow.enableInputMethods(false);
        this.compositionWindow.pack();
        Dimension size = this.compositionWindow.getSize();
        Dimension screenSize = this.getToolkit().getScreenSize();
        this.compositionWindow.setLocation(screenSize.width - size.width - 20, screenSize.height - size.height - 100);
        this.compositionWindow.setVisible(false);
    }
    
    synchronized void setHandlerInfo(CompositionAreaHandler handler, InputContext inputContext) {
        this.handler = handler;
        ((InputMethodWindow)this.compositionWindow).setInputContext(inputContext);
    }
    
    @Override
    public InputMethodRequests getInputMethodRequests() {
        return this.handler;
    }
    
    private Rectangle getCaretRectangle(TextHitInfo hit) {
        int round = 0;
        TextLayout composedTextLayout = this.composedTextLayout;
        if (composedTextLayout != null) {
            round = Math.round(composedTextLayout.getCaretInfo(hit)[0]);
        }
        Graphics graphics = this.getGraphics();
        FontMetrics fontMetrics = null;
        try {
            fontMetrics = graphics.getFontMetrics();
        }
        finally {
            graphics.dispose();
        }
        return new Rectangle(5 + round, 15 - fontMetrics.getAscent(), 0, fontMetrics.getAscent() + fontMetrics.getDescent());
    }
    
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        g.setColor(this.getForeground());
        TextLayout composedTextLayout = this.composedTextLayout;
        if (composedTextLayout != null) {
            composedTextLayout.draw((Graphics2D)g, 5.0f, 15.0f);
        }
        if (this.caret != null) {
            Rectangle caretRectangle = this.getCaretRectangle(this.caret);
            g.setXORMode(this.getBackground());
            g.fillRect(caretRectangle.x, caretRectangle.y, 1, caretRectangle.height);
            g.setPaintMode();
        }
    }
    
    void setCompositionAreaVisible(boolean visible) {
        this.compositionWindow.setVisible(visible);
    }
    
    boolean isCompositionAreaVisible() {
        return this.compositionWindow.isVisible();
    }
    
    @Override
    public void inputMethodTextChanged(InputMethodEvent inputMethodEvent) {
        this.handler.inputMethodTextChanged(inputMethodEvent);
    }
    
    @Override
    public void caretPositionChanged(InputMethodEvent inputMethodEvent) {
        this.handler.caretPositionChanged(inputMethodEvent);
    }
    
    void setText(AttributedCharacterIterator text, TextHitInfo caret) {
        this.composedTextLayout = null;
        if (text == null) {
            this.compositionWindow.setVisible(false);
            this.caret = null;
        }
        else {
            if (!this.compositionWindow.isVisible()) {
                this.compositionWindow.setVisible(true);
            }
            Graphics graphics = this.getGraphics();
            if (graphics == null) {
                return;
            }
            try {
                this.updateWindowLocation();
                this.composedTextLayout = new TextLayout(text, ((Graphics2D)graphics).getFontRenderContext());
                Rectangle2D bounds = this.composedTextLayout.getBounds();
                this.caret = caret;
                int height = (int)graphics.getFontMetrics().getMaxCharBounds(graphics).getHeight() + 3;
                int height2 = height + this.compositionWindow.getInsets().top + this.compositionWindow.getInsets().bottom;
                int width = (this.handler.getClientInputMethodRequests() == null) ? 480 : ((int)bounds.getWidth() + 10);
                int width2 = width + this.compositionWindow.getInsets().left + this.compositionWindow.getInsets().right;
                this.setPreferredSize(new Dimension(width, height));
                this.compositionWindow.setSize(new Dimension(width2, height2));
                this.paint(graphics);
            }
            finally {
                graphics.dispose();
            }
        }
    }
    
    void setCaret(TextHitInfo caret) {
        this.caret = caret;
        if (this.compositionWindow.isVisible()) {
            Graphics graphics = this.getGraphics();
            try {
                this.paint(graphics);
            }
            finally {
                graphics.dispose();
            }
        }
    }
    
    void updateWindowLocation() {
        InputMethodRequests clientInputMethodRequests = this.handler.getClientInputMethodRequests();
        if (clientInputMethodRequests == null) {
            return;
        }
        Point location = new Point();
        Rectangle textLocation = clientInputMethodRequests.getTextLocation(null);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension size = this.compositionWindow.getSize();
        if (textLocation.x + size.width > screenSize.width) {
            location.x = screenSize.width - size.width;
        }
        else {
            location.x = textLocation.x;
        }
        if (textLocation.y + textLocation.height + 2 + size.height > screenSize.height) {
            location.y = textLocation.y - 2 - size.height;
        }
        else {
            location.y = textLocation.y + textLocation.height + 2;
        }
        this.compositionWindow.setLocation(location);
        return;
    }
    
    Rectangle getTextLocation(TextHitInfo textHitInfo) {
        Rectangle caretRectangle = this.getCaretRectangle(textHitInfo);
        Point locationOnScreen = this.getLocationOnScreen();
        caretRectangle.translate(locationOnScreen.x, locationOnScreen.y);
        return caretRectangle;
    }
    
    TextHitInfo getLocationOffset(int n, int n2) {
        TextLayout composedTextLayout = this.composedTextLayout;
        if (composedTextLayout == null) {
            return null;
        }
        Point locationOnScreen = this.getLocationOnScreen();
        n -= locationOnScreen.x + 5;
        n2 -= locationOnScreen.y + 15;
        if (composedTextLayout.getBounds().contains((double)n, (double)n2)) {
            return composedTextLayout.hitTestChar((float)n, (float)n2);
        }
        return null;
    }
    
    void setCompositionAreaUndecorated(boolean undecorated) {
        if (this.compositionWindow.isDisplayable()) {
            this.compositionWindow.removeNotify();
        }
        this.compositionWindow.setUndecorated(undecorated);
        this.compositionWindow.pack();
    }
    
    class FrameWindowAdapter extends WindowAdapter
    {
        @Override
        public void windowActivated(WindowEvent windowEvent) {
            CompositionArea.this.requestFocus();
        }
    }
}
